'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.createMapper = createMapper;

var _httpMethods = require('./http-methods');

var _httpMethods2 = _interopRequireDefault(_httpMethods);

var _pathBuilder = require('../utils/path-builder');

var _pathBuilder2 = _interopRequireDefault(_pathBuilder);

var _request = require('./request');

var _request2 = _interopRequireDefault(_request);

var _stateDispatcher = require('./state-dispatcher');

var _stateDispatcher2 = _interopRequireDefault(_stateDispatcher);

var _actionBuilder = require('../reducer/action-builder');

var _actionBuilder2 = _interopRequireDefault(_actionBuilder);

var _httpLayers = require('../http-layers');

var _merge = require('../utils/merge');

var _merge2 = _interopRequireDefault(_merge);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Create an api object for mapping an api
 * @param {Object} store - redux store
 * @param {Object} config - Configuration object
 * @param {Object} [httpLayer] - http layer object
 * @param {Function} [transformer] - a transformer function that takes the http layer response and transform into something else
 * @returns {{store: *, name: String, httpLayer: Object, host: String, headers: Object.<String, (String|Function)>, options: Object}}
 */
function createMapper(store, config, httpLayer, transformer) {
  var mapper = {
    store: store,
    name: config.name,
    httpLayer: httpLayer ? httpLayer : new _httpLayers.FetchHttpLayer(),
    host: config.host,
    params: config.params,
    headers: config.headers,
    options: config.options,
    transformer: transformer || function (httpLayerResponse) {
      return httpLayerResponse;
    }
  };

  var resources = config.resources;

  resources.forEach(function (resource) {
    addResourceToMapper(mapper, resource);
  });

  return mapper;
}

function addResourceToMapper(mapper, resource) {
  var mResource = {
    _mapper: mapper,
    name: resource.name,
    path: resource.path,
    params: resource.params,
    headers: resource.headers,
    options: resource.options
  };

  var endPoints = resource.endPoints;

  endPoints.forEach(function (endPoint) {
    addEndPointToResource(mResource, endPoint);
  });

  mapper[resource.name] = mResource;
}

function addEndPointToResource(resource, endPoint) {
  var mEndPoint = {
    _resource: resource,
    name: endPoint.name,
    path: endPoint.path,
    params: endPoint.params,
    headers: endPoint.headers,
    options: endPoint.options,
    method: endPoint.method ? endPoint.method : _httpMethods2.default.GET,
    action: endPoint.action
  };

  var actionBuilder = (0, _actionBuilder2.default)(resource.name, mEndPoint.name, resource._mapper.name);

  resource[endPoint.name] = buildEndPointFunc(mEndPoint);
  resource[endPoint.name].path = mEndPoint.path;
  resource[endPoint.name].headers = mEndPoint.headers;
  resource[endPoint.name].options = mEndPoint.options;
  resource[endPoint.name].method = mEndPoint.method;
  resource[endPoint.name].action = mEndPoint.action;
  resource[endPoint.name].params = mEndPoint.params;
  resource[endPoint.name].clearState = function () {
    var dispatch = resource._mapper.store.dispatch;

    dispatch({ type: actionBuilder("CLEAR_STATE") });
  };
  resource[endPoint.name].dispatch = function (action) {
    if (action.type) action.type = actionBuilder(action.type);

    resource._mapper.store.dispatch(action);
  };
}

function buildEndPointFunc(mEndPoint) {
  return function (params, body, headers, options) {
    var mapper = mEndPoint._resource._mapper;
    var transformer = mapper.transformer;
    var resource = mEndPoint._resource;
    var endPoint = resource[mEndPoint.name];
    var httpLayer = mEndPoint._resource._mapper.httpLayer;
    var store = mapper.store;

    var ePath = endPoint.path;
    var eHeaders = endPoint.headers;
    var eOptions = endPoint.options;
    var eAction = endPoint.action;
    var eMethod = endPoint.method;

    var mapperParams = mapper.params || {};
    var resourceParams = resource.params || {};
    var endPointParams = endPoint.params || {};
    var requestParams = (0, _merge2.default)(mapperParams, resourceParams, endPointParams, params);

    var fullPath = (0, _pathBuilder2.default)(mapper.host, resource.path, ePath, requestParams);

    headers = (0, _merge2.default)(mapper.headers, resource.headers, eHeaders, headers);

    if (httpLayer.mergeOptions) options = httpLayer.mergeOptions(mapper.options, resource.options, eOptions, options);else options = _extends({}, mapper.options, resource.options, eOptions, options);

    var request = (0, _request2.default)(fullPath, eMethod, requestParams, headers, body, mapper, resource, endPoint, options);

    var stateDispatcher = (0, _stateDispatcher2.default)(store, eAction, mEndPoint);

    if (!httpLayer[eMethod]) throw new Error('Current http-layers do not support ' + eMethod + ' method');

    return transformer(httpLayer[eMethod](stateDispatcher, request));
  };
}